//+------------------------------------------------------------------
//|
//| Derivative Oscillator as published by Constance Brown
//| "Technical Analysis for the trading professional"
//| Low lag oscillator which is useful at divergence analysis.
//+------------------------------------------------------------------
#property copyright "www.forex-tsd.com"
#property link      "www.forex-tsd.com"

#property indicator_separate_window
#property indicator_buffers 3
#property indicator_color1  DeepSkyBlue
#property indicator_color2  PaleVioletRed
#property indicator_color3  DimGray
#property indicator_width1  2
#property indicator_width2  2
#property indicator_width3  2

//
//
//
//
//

extern string TimeFrame          = "Current time frame";
extern int    RsiPeriod          = 14;
extern int    RsiPrice           = PRICE_CLOSE;
extern int    EmaSmoothing1      = 15;
extern int    EmaSmoothing2      =  3;
extern int    SmaSmoothing       =  9;
extern bool   alertsOn           = true;
extern bool   alertsOnCurrent    = true;
extern bool   alertsMessage      = true;
extern bool   alertsNotify       = false;
extern bool   alertsSound        = false;
extern bool   alertsEmail        = false;
extern bool   Interpolate        = false;

extern bool   arrowsVisible      = true;
extern string arrowsIdentifier   = "derivative osc Arrows1";
extern double arrowsDisplacement = 1.0;
extern color  arrowsUpColor      = DeepSkyBlue;
extern color  arrowsDnColor      = PaleVioletRed;
extern int    arrowsUpCode       = 241;
extern int    arrowsDownCode     = 242;

//
//
//
//
//

double der[];
double derUu[];
double derUd[];
double derDd[];
double derDu[];
double trend[];
double slope[];

string indicatorFileName;
bool   returnBars;
bool   calculateValue;
int    timeFrame;

//+------------------------------------------------------------------
//|
//+------------------------------------------------------------------
//
//
//
//
//

int init()
{
   IndicatorDigits(5);
   IndicatorBuffers(4);
      SetIndexBuffer(0,derUu); SetIndexStyle(0,DRAW_HISTOGRAM);
      SetIndexBuffer(1,derDu); SetIndexStyle(1,DRAW_HISTOGRAM);
      SetIndexBuffer(2,der);
      SetIndexBuffer(3,trend);

      //
      //
      //
      //
      //
            
      indicatorFileName = WindowExpertName();
      returnBars        = (TimeFrame=="returnBars");     if (returnBars)     return(0);
      calculateValue    = (TimeFrame=="calculateValue"); if (calculateValue) return(0);
      timeFrame         = stringToTimeFrame(TimeFrame);
   IndicatorShortName(timeFrameToString(timeFrame)+" Derivative oscillator ("+RsiPeriod+","+EmaSmoothing1+","+EmaSmoothing2+","+SmaSmoothing+")");
   return(0);
}

int deinit()
{  
   if (!calculateValue && arrowsVisible) deleteArrows();  
   
return(0); 
}

//+------------------------------------------------------------------
//|
//+------------------------------------------------------------------
//
//
// 
//
// 
//
//

double work[][2];
#define _ema1 0
#define _ema2 1

//
//
//
//
//

int start() 
{
   int i,k,n,r,limit,counted_bars = IndicatorCounted();

      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
            limit = MathMin(Bars-counted_bars,Bars-1);
            if (returnBars) { derUu[0] = limit; return(0); }
   	
   //
   //
   //
   //
   //
      	
   if (calculateValue || timeFrame==Period())
   {      	
      if (ArrayRange(work,0)!=Bars) ArrayResize(work,Bars);
      double alpha1 = 2.0/(1.0+EmaSmoothing1);
      double alpha2 = 2.0/(1.0+EmaSmoothing2);
      
      //
      //
      //
      //
      //
      
      for(i=limit, r=Bars-i-1; i>=0; i--,r++)
      {
         double rsi = iRSI(NULL,0,RsiPeriod,RsiPrice,i);
                      work[r][_ema1] = work[r-1][_ema1]+alpha1*(rsi           -work[r-1][_ema1]);
                      work[r][_ema2] = work[r-1][_ema2]+alpha2*(work[r][_ema1]-work[r-1][_ema2]);
         double sma = work[r][_ema2]; for (k=1; k<SmaSmoothing && (r-k)>=0; k++) sma += work[r-k][_ema2]; sma /= k;

         //
         //
         //
         //
         //
      
         der[i] = work[r][_ema2]-sma;
         derUu[i] = EMPTY_VALUE;
         derDu[i] = EMPTY_VALUE;
         trend[i] = trend[i+1];
            if (der[i]>0)      trend[i] =  1;
            if (der[i]<0)      trend[i] = -1;
            if (trend[i] == 1) derUu[i] = der[i];
            if (trend[i] ==-1) derDu[i] = der[i];
            manageArrow(i);
      }
      manageAlerts();
      return(0);
   }
   
   //
   //
   //
   //
   //
   
   limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   for(i=limit; i>=0; i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         der[i]   = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",RsiPeriod,RsiPrice,EmaSmoothing1,EmaSmoothing2,SmaSmoothing,alertsOn,alertsOnCurrent,alertsMessage,alertsNotify,alertsSound,alertsEmail,2,y);
         trend[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",RsiPeriod,RsiPrice,EmaSmoothing1,EmaSmoothing2,SmaSmoothing,alertsOn,alertsOnCurrent,alertsMessage,alertsNotify,alertsSound,alertsEmail,3,y);   
         derUu[i] = EMPTY_VALUE;
         derDu[i] = EMPTY_VALUE;
         manageArrow(i);
          
         //
         //
         //
         //
         //
      
         if (!Interpolate || y==iBarShift(NULL,timeFrame,Time[i-1])) continue;

         //
         //
         //
         //
         //

         datetime time = iTime(NULL,timeFrame,y);
            for(n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
            for(k = 1; k < n; k++)  der[i+k] = der[i] + (der[i+n]-der[i])*k/n;
   }
   for(i=limit; i>=0; i--)
   { 
      if (trend[i] == 1) derUu[i] = der[i];
      if (trend[i] ==-1) derDu[i] = der[i];
      manageArrow(i);
   }                         
   return(0);
}



//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int tchar = StringGetChar(s, length);
         if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
                     s = StringSetChar(s, length, tchar - 32);
         else if(tchar > -33 && tchar < 0)
                     s = StringSetChar(s, length, tchar + 224);
   }
   return(s);
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1;
      if (trend[whichBar] != trend[whichBar+1])
         {
            if (trend[whichBar] ==  1) doAlert(whichBar," crossing zero up");
            if (trend[whichBar] == -1) doAlert(whichBar," crossing zero down");
         }
   }
}

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," - ",timeFrameToString(timeFrame)+" Derivative oscillator ",doWhat);
          if (alertsMessage) Alert(message);
          if (alertsNotify)  SendNotification(message);
          if (alertsEmail)   SendMail(StringConcatenate(Symbol()," Derivative oscillator "),message);
          if (alertsSound)   PlaySound("alert2.wav");
   }
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void manageArrow(int i)
{
   if (arrowsVisible)
   {
      deleteArrow(Time[i]);
      if (trend[i] != trend[i+1])
      {
         if (trend[i] == 1) drawArrow(i,arrowsUpColor,arrowsUpCode  ,false);
         if (trend[i] ==-1) drawArrow(i,arrowsDnColor,arrowsDownCode,true);
      }
   }
}               

//
//
//
//
//

//
//
//
//
//

void drawArrow(int i,color theColor,int theCode,bool up)
{
   string name = arrowsIdentifier+":"+Time[i];
   double gap  = iATR(NULL,0,20,i);   
   
      //
      //
      //
      //
      //
      
      ObjectCreate(name,OBJ_ARROW,0,Time[i],0);
         ObjectSet(name,OBJPROP_ARROWCODE,theCode);
         ObjectSet(name,OBJPROP_COLOR,theColor);
         if (up)
               ObjectSet(name,OBJPROP_PRICE1,High[i] + arrowsDisplacement * gap);
         else  ObjectSet(name,OBJPROP_PRICE1,Low[i]  - arrowsDisplacement * gap);
}

//
//
//
//
//

//
//
//
//
//

void deleteArrows()
{
   string lookFor       = arrowsIdentifier+":";
   int    lookForLength = StringLen(lookFor);
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string objectName = ObjectName(i);
         if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
   }
}

//
//
//
//
//

void deleteArrow(datetime time)
{
   string lookFor = arrowsIdentifier+":"+time; ObjectDelete(lookFor);
}

